# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import employee_list_params, employee_create_params, employee_update_params
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.employee import Employee

__all__ = ["EmployeesResource", "AsyncEmployeesResource"]


class EmployeesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EmployeesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return EmployeesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmployeesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return EmployeesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        conductor_end_user_id: str,
        account_number: str | NotGiven = NOT_GIVEN,
        additional_notes: Iterable[employee_create_params.AdditionalNote] | NotGiven = NOT_GIVEN,
        address: employee_create_params.Address | NotGiven = NOT_GIVEN,
        adjusted_service_date: Union[str, date] | NotGiven = NOT_GIVEN,
        alternate_phone: str | NotGiven = NOT_GIVEN,
        billing_rate_id: str | NotGiven = NOT_GIVEN,
        birth_date: Union[str, date] | NotGiven = NOT_GIVEN,
        custom_contact_fields: Iterable[employee_create_params.CustomContactField] | NotGiven = NOT_GIVEN,
        department: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        disability_description: str | NotGiven = NOT_GIVEN,
        disability_status: Literal["disabled", "non_disabled"] | NotGiven = NOT_GIVEN,
        email: str | NotGiven = NOT_GIVEN,
        emergency_contact: employee_create_params.EmergencyContact | NotGiven = NOT_GIVEN,
        employee_payroll: employee_create_params.EmployeePayroll | NotGiven = NOT_GIVEN,
        employee_type: Literal["officer", "owner", "regular", "statutory"] | NotGiven = NOT_GIVEN,
        employment_status: Literal["full_time", "part_time"] | NotGiven = NOT_GIVEN,
        ethnicity: Literal["american_indian", "asian", "black", "hawaiian", "hispanic", "white", "two_or_more_races"]
        | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        fax: str | NotGiven = NOT_GIVEN,
        first_name: str | NotGiven = NOT_GIVEN,
        gender: Literal["male", "female"] | NotGiven = NOT_GIVEN,
        hired_date: Union[str, date] | NotGiven = NOT_GIVEN,
        i9_on_file_status: Literal["on_file", "not_on_file"] | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        job_title: str | NotGiven = NOT_GIVEN,
        key_employee_status: Literal["key_employee", "non_key_employee"] | NotGiven = NOT_GIVEN,
        last_name: str | NotGiven = NOT_GIVEN,
        middle_name: str | NotGiven = NOT_GIVEN,
        military_status: Literal["active", "reserve"] | NotGiven = NOT_GIVEN,
        mobile: str | NotGiven = NOT_GIVEN,
        note: str | NotGiven = NOT_GIVEN,
        original_hire_date: Union[str, date] | NotGiven = NOT_GIVEN,
        overtime_exempt_status: Literal["exempt", "non_exempt"] | NotGiven = NOT_GIVEN,
        pager: str | NotGiven = NOT_GIVEN,
        pager_pin: str | NotGiven = NOT_GIVEN,
        phone: str | NotGiven = NOT_GIVEN,
        print_as: str | NotGiven = NOT_GIVEN,
        salutation: str | NotGiven = NOT_GIVEN,
        ssn: str | NotGiven = NOT_GIVEN,
        supervisor_id: str | NotGiven = NOT_GIVEN,
        target_bonus: str | NotGiven = NOT_GIVEN,
        termination_date: Union[str, date] | NotGiven = NOT_GIVEN,
        us_citizenship_status: Literal["citizen", "non_citizen"] | NotGiven = NOT_GIVEN,
        us_veteran_status: Literal["veteran", "non_veteran"] | NotGiven = NOT_GIVEN,
        work_authorization_expiration_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Employee:
        """
        Creates a new employee.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_number: The employee's account number, which appears in the QuickBooks chart of
              accounts, reports, and graphs.

              Note that if the "Use Account Numbers" preference is turned off in QuickBooks,
              the account number may not be visible in the user interface, but it can still be
              set and retrieved through the API.

          additional_notes: Additional notes about this employee.

          address: The employee's address.

              If the company uses QuickBooks Payroll for this employee, this address must
              specify a complete address, including city, state, ZIP (or postal) code, and at
              least one line of the street address.

          adjusted_service_date: The adjusted service date for this employee, in ISO 8601 format (YYYY-MM-DD).
              This date accounts for previous employment periods or leaves that affect
              seniority.

          alternate_phone: The employee's alternate telephone number.

          billing_rate_id: The employee's billing rate, used to override service item rates in time
              tracking activities.

          birth_date: This employee's date of birth, in ISO 8601 format (YYYY-MM-DD).

          custom_contact_fields: Additional custom contact fields for this employee, such as phone numbers or
              email addresses.

          department: The employee's department. Found in the "employment job details" section of the
              employee's record in QuickBooks.

          description: A description of this employee. Found in the "employment job details" section of
              the employee's record in QuickBooks.

          disability_description: A description of this employee's disability.

          disability_status: Indicates whether this employee is disabled.

          email: The employee's email address.

          emergency_contact: The employee's emergency contacts.

          employee_payroll: The employee's payroll information.

          employee_type: The employee type. This affects payroll taxes - a statutory employee is defined
              as an employee by statute. Note that owners/partners are typically on the "Other
              Names" list in QuickBooks, but if listed as an employee their type will be
              `owner`.

          employment_status: Indicates whether this employee is a part-time or full-time employee.

          ethnicity: This employee's ethnicity.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          fax: The employee's fax number.

          first_name: The employee's first name.

              Maximum length: 25 characters.

          gender: This employee's gender.

          hired_date: The date this employee was hired, in ISO 8601 format (YYYY-MM-DD).

          i9_on_file_status: Indicates whether this employee's I-9 is on file.

          is_active: Indicates whether this employee is active. Inactive objects are typically hidden
              from views and reports in QuickBooks. Defaults to `true`.

          job_title: The employee's job title.

          key_employee_status: Indicates whether this employee is a key employee.

          last_name: The employee's last name.

              Maximum length: 25 characters.

          middle_name: The employee's middle name.

              Maximum length: 5 characters.

          military_status: This employee's military status if they are a U.S. veteran.

          mobile: The employee's mobile phone number.

          note: A note or comment about this employee.

          original_hire_date: The original hire date for this employee, in ISO 8601 format (YYYY-MM-DD).

          overtime_exempt_status: Indicates whether this employee is exempt from overtime pay. This classification
              is based on U.S. labor laws (FLSA).

          pager: The employee's pager number.

          pager_pin: The employee's pager PIN.

          phone: The employee's primary telephone number.

          print_as: The name to use when printing this employee from QuickBooks. By default, this is
              the same as the `name` field.

          salutation: The employee's formal salutation title that precedes their name, such as "Mr.",
              "Ms.", or "Dr.".

          ssn: The employee's Social Security Number. The value can be with or without dashes.

              **NOTE**: This field cannot be changed after the employee is created.

          supervisor_id: The employee's supervisor. Found in the "employment job details" section of the
              employee's record in QuickBooks.

          target_bonus: The target bonus for this employee, represented as a decimal string. Found in
              the "employment job details" section of the employee's record in QuickBooks.

          termination_date: The date this employee's employment ended with the company, in ISO 8601 format
              (YYYY-MM-DD). This is also known as the released date or separation date.

          us_citizenship_status: Indicates whether this employee is a U.S. citizen.

          us_veteran_status: Indicates whether this employee is a U.S. veteran.

          work_authorization_expiration_date: The date this employee's work authorization expires, in ISO 8601 format
              (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/employees",
            body=maybe_transform(
                {
                    "account_number": account_number,
                    "additional_notes": additional_notes,
                    "address": address,
                    "adjusted_service_date": adjusted_service_date,
                    "alternate_phone": alternate_phone,
                    "billing_rate_id": billing_rate_id,
                    "birth_date": birth_date,
                    "custom_contact_fields": custom_contact_fields,
                    "department": department,
                    "description": description,
                    "disability_description": disability_description,
                    "disability_status": disability_status,
                    "email": email,
                    "emergency_contact": emergency_contact,
                    "employee_payroll": employee_payroll,
                    "employee_type": employee_type,
                    "employment_status": employment_status,
                    "ethnicity": ethnicity,
                    "external_id": external_id,
                    "fax": fax,
                    "first_name": first_name,
                    "gender": gender,
                    "hired_date": hired_date,
                    "i9_on_file_status": i9_on_file_status,
                    "is_active": is_active,
                    "job_title": job_title,
                    "key_employee_status": key_employee_status,
                    "last_name": last_name,
                    "middle_name": middle_name,
                    "military_status": military_status,
                    "mobile": mobile,
                    "note": note,
                    "original_hire_date": original_hire_date,
                    "overtime_exempt_status": overtime_exempt_status,
                    "pager": pager,
                    "pager_pin": pager_pin,
                    "phone": phone,
                    "print_as": print_as,
                    "salutation": salutation,
                    "ssn": ssn,
                    "supervisor_id": supervisor_id,
                    "target_bonus": target_bonus,
                    "termination_date": termination_date,
                    "us_citizenship_status": us_citizenship_status,
                    "us_veteran_status": us_veteran_status,
                    "work_authorization_expiration_date": work_authorization_expiration_date,
                },
                employee_create_params.EmployeeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Employee,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Employee:
        """
        Retrieves an employee by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the employee to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/employees/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Employee,
        )

    def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        account_number: str | NotGiven = NOT_GIVEN,
        additional_notes: Iterable[employee_update_params.AdditionalNote] | NotGiven = NOT_GIVEN,
        address: employee_update_params.Address | NotGiven = NOT_GIVEN,
        adjusted_service_date: Union[str, date] | NotGiven = NOT_GIVEN,
        alternate_phone: str | NotGiven = NOT_GIVEN,
        billing_rate_id: str | NotGiven = NOT_GIVEN,
        birth_date: Union[str, date] | NotGiven = NOT_GIVEN,
        custom_contact_fields: Iterable[employee_update_params.CustomContactField] | NotGiven = NOT_GIVEN,
        department: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        disability_description: str | NotGiven = NOT_GIVEN,
        disability_status: Literal["disabled", "non_disabled"] | NotGiven = NOT_GIVEN,
        email: str | NotGiven = NOT_GIVEN,
        emergency_contact: employee_update_params.EmergencyContact | NotGiven = NOT_GIVEN,
        employee_payroll: employee_update_params.EmployeePayroll | NotGiven = NOT_GIVEN,
        employee_type: Literal["officer", "owner", "regular", "statutory"] | NotGiven = NOT_GIVEN,
        employment_status: Literal["full_time", "part_time"] | NotGiven = NOT_GIVEN,
        ethnicity: Literal["american_indian", "asian", "black", "hawaiian", "hispanic", "white", "two_or_more_races"]
        | NotGiven = NOT_GIVEN,
        fax: str | NotGiven = NOT_GIVEN,
        first_name: str | NotGiven = NOT_GIVEN,
        hired_date: Union[str, date] | NotGiven = NOT_GIVEN,
        i9_on_file_status: Literal["on_file", "not_on_file"] | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        job_title: str | NotGiven = NOT_GIVEN,
        key_employee_status: Literal["key_employee", "non_key_employee"] | NotGiven = NOT_GIVEN,
        last_name: str | NotGiven = NOT_GIVEN,
        middle_name: str | NotGiven = NOT_GIVEN,
        military_status: Literal["active", "reserve"] | NotGiven = NOT_GIVEN,
        mobile: str | NotGiven = NOT_GIVEN,
        note: str | NotGiven = NOT_GIVEN,
        original_hire_date: Union[str, date] | NotGiven = NOT_GIVEN,
        overtime_exempt_status: Literal["exempt", "non_exempt"] | NotGiven = NOT_GIVEN,
        pager: str | NotGiven = NOT_GIVEN,
        pager_pin: str | NotGiven = NOT_GIVEN,
        phone: str | NotGiven = NOT_GIVEN,
        print_as: str | NotGiven = NOT_GIVEN,
        salutation: str | NotGiven = NOT_GIVEN,
        supervisor_id: str | NotGiven = NOT_GIVEN,
        target_bonus: str | NotGiven = NOT_GIVEN,
        termination_date: Union[str, date] | NotGiven = NOT_GIVEN,
        us_citizenship_status: Literal["citizen", "non_citizen"] | NotGiven = NOT_GIVEN,
        us_veteran_status: Literal["veteran", "non_veteran"] | NotGiven = NOT_GIVEN,
        work_authorization_expiration_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Employee:
        """
        Updates an existing employee.

        Args:
          id: The QuickBooks-assigned unique identifier of the employee to update.

          revision_number: The current QuickBooks-assigned revision number of the employee object you are
              updating, which you can get by fetching the object first. Provide the most
              recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_number: The employee's account number, which appears in the QuickBooks chart of
              accounts, reports, and graphs.

              Note that if the "Use Account Numbers" preference is turned off in QuickBooks,
              the account number may not be visible in the user interface, but it can still be
              set and retrieved through the API.

          additional_notes: Additional notes about this employee.

          address: The employee's address.

              If the company uses QuickBooks Payroll for this employee, this address must
              specify a complete address, including city, state, ZIP (or postal) code, and at
              least one line of the street address.

          adjusted_service_date: The adjusted service date for this employee, in ISO 8601 format (YYYY-MM-DD).
              This date accounts for previous employment periods or leaves that affect
              seniority.

          alternate_phone: The employee's alternate telephone number.

          billing_rate_id: The employee's billing rate, used to override service item rates in time
              tracking activities.

          birth_date: This employee's date of birth, in ISO 8601 format (YYYY-MM-DD).

          custom_contact_fields: Additional custom contact fields for this employee, such as phone numbers or
              email addresses.

          department: The employee's department. Found in the "employment job details" section of the
              employee's record in QuickBooks.

          description: A description of this employee. Found in the "employment job details" section of
              the employee's record in QuickBooks.

          disability_description: A description of this employee's disability.

          disability_status: Indicates whether this employee is disabled.

          email: The employee's email address.

          emergency_contact: The employee's emergency contacts.

          employee_payroll: The employee's payroll information.

          employee_type: The employee type. This affects payroll taxes - a statutory employee is defined
              as an employee by statute. Note that owners/partners are typically on the "Other
              Names" list in QuickBooks, but if listed as an employee their type will be
              `owner`.

          employment_status: Indicates whether this employee is a part-time or full-time employee.

          ethnicity: This employee's ethnicity.

          fax: The employee's fax number.

          first_name: The employee's first name.

              Maximum length: 25 characters.

          hired_date: The date this employee was hired, in ISO 8601 format (YYYY-MM-DD).

          i9_on_file_status: Indicates whether this employee's I-9 is on file.

          is_active: Indicates whether this employee is active. Inactive objects are typically hidden
              from views and reports in QuickBooks. Defaults to `true`.

          job_title: The employee's job title.

          key_employee_status: Indicates whether this employee is a key employee.

          last_name: The employee's last name.

              Maximum length: 25 characters.

          middle_name: The employee's middle name.

              Maximum length: 5 characters.

          military_status: This employee's military status if they are a U.S. veteran.

          mobile: The employee's mobile phone number.

          note: A note or comment about this employee.

          original_hire_date: The original hire date for this employee, in ISO 8601 format (YYYY-MM-DD).

          overtime_exempt_status: Indicates whether this employee is exempt from overtime pay. This classification
              is based on U.S. labor laws (FLSA).

          pager: The employee's pager number.

          pager_pin: The employee's pager PIN.

          phone: The employee's primary telephone number.

          print_as: The name to use when printing this employee from QuickBooks. By default, this is
              the same as the `name` field.

          salutation: The employee's formal salutation title that precedes their name, such as "Mr.",
              "Ms.", or "Dr.".

          supervisor_id: The employee's supervisor. Found in the "employment job details" section of the
              employee's record in QuickBooks.

          target_bonus: The target bonus for this employee, represented as a decimal string. Found in
              the "employment job details" section of the employee's record in QuickBooks.

          termination_date: The date this employee's employment ended with the company, in ISO 8601 format
              (YYYY-MM-DD). This is also known as the released date or separation date.

          us_citizenship_status: Indicates whether this employee is a U.S. citizen.

          us_veteran_status: Indicates whether this employee is a U.S. veteran.

          work_authorization_expiration_date: The date this employee's work authorization expires, in ISO 8601 format
              (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/employees/{id}",
            body=maybe_transform(
                {
                    "revision_number": revision_number,
                    "account_number": account_number,
                    "additional_notes": additional_notes,
                    "address": address,
                    "adjusted_service_date": adjusted_service_date,
                    "alternate_phone": alternate_phone,
                    "billing_rate_id": billing_rate_id,
                    "birth_date": birth_date,
                    "custom_contact_fields": custom_contact_fields,
                    "department": department,
                    "description": description,
                    "disability_description": disability_description,
                    "disability_status": disability_status,
                    "email": email,
                    "emergency_contact": emergency_contact,
                    "employee_payroll": employee_payroll,
                    "employee_type": employee_type,
                    "employment_status": employment_status,
                    "ethnicity": ethnicity,
                    "fax": fax,
                    "first_name": first_name,
                    "hired_date": hired_date,
                    "i9_on_file_status": i9_on_file_status,
                    "is_active": is_active,
                    "job_title": job_title,
                    "key_employee_status": key_employee_status,
                    "last_name": last_name,
                    "middle_name": middle_name,
                    "military_status": military_status,
                    "mobile": mobile,
                    "note": note,
                    "original_hire_date": original_hire_date,
                    "overtime_exempt_status": overtime_exempt_status,
                    "pager": pager,
                    "pager_pin": pager_pin,
                    "phone": phone,
                    "print_as": print_as,
                    "salutation": salutation,
                    "supervisor_id": supervisor_id,
                    "target_bonus": target_bonus,
                    "termination_date": termination_date,
                    "us_citizenship_status": us_citizenship_status,
                    "us_veteran_status": us_veteran_status,
                    "work_authorization_expiration_date": work_authorization_expiration_date,
                },
                employee_update_params.EmployeeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Employee,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        names: List[str] | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[Employee]:
        """Returns a list of employees.

        Use the `cursor` parameter to paginate through the
        results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          ids: Filter for specific employees by their QuickBooks-assigned unique identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          name_contains: Filter for employees whose `name` contains this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameStartsWith` or
              `nameEndsWith`.

          name_ends_with: Filter for employees whose `name` ends with this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameContains` or
              `nameStartsWith`.

          name_from: Filter for employees whose `name` is alphabetically greater than or equal to
              this value.

          names: Filter for specific employees by their name(s), case-insensitive. Like `id`,
              `name` is a unique identifier for an employee.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          name_starts_with: Filter for employees whose `name` starts with this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameContains` or
              `nameEndsWith`.

          name_to: Filter for employees whose `name` is alphabetically less than or equal to this
              value.

          status: Filter for employees that are active, inactive, or both.

          updated_after: Filter for employees updated on or after this date and time, in ISO 8601 format
              (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
              assumed to be 00:00:00 of that day.

          updated_before: Filter for employees updated on or before this date and time, in ISO 8601 format
              (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
              assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/employees",
            page=SyncCursorPage[Employee],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "names": names,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    employee_list_params.EmployeeListParams,
                ),
            ),
            model=Employee,
        )


class AsyncEmployeesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEmployeesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmployeesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmployeesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncEmployeesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        conductor_end_user_id: str,
        account_number: str | NotGiven = NOT_GIVEN,
        additional_notes: Iterable[employee_create_params.AdditionalNote] | NotGiven = NOT_GIVEN,
        address: employee_create_params.Address | NotGiven = NOT_GIVEN,
        adjusted_service_date: Union[str, date] | NotGiven = NOT_GIVEN,
        alternate_phone: str | NotGiven = NOT_GIVEN,
        billing_rate_id: str | NotGiven = NOT_GIVEN,
        birth_date: Union[str, date] | NotGiven = NOT_GIVEN,
        custom_contact_fields: Iterable[employee_create_params.CustomContactField] | NotGiven = NOT_GIVEN,
        department: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        disability_description: str | NotGiven = NOT_GIVEN,
        disability_status: Literal["disabled", "non_disabled"] | NotGiven = NOT_GIVEN,
        email: str | NotGiven = NOT_GIVEN,
        emergency_contact: employee_create_params.EmergencyContact | NotGiven = NOT_GIVEN,
        employee_payroll: employee_create_params.EmployeePayroll | NotGiven = NOT_GIVEN,
        employee_type: Literal["officer", "owner", "regular", "statutory"] | NotGiven = NOT_GIVEN,
        employment_status: Literal["full_time", "part_time"] | NotGiven = NOT_GIVEN,
        ethnicity: Literal["american_indian", "asian", "black", "hawaiian", "hispanic", "white", "two_or_more_races"]
        | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        fax: str | NotGiven = NOT_GIVEN,
        first_name: str | NotGiven = NOT_GIVEN,
        gender: Literal["male", "female"] | NotGiven = NOT_GIVEN,
        hired_date: Union[str, date] | NotGiven = NOT_GIVEN,
        i9_on_file_status: Literal["on_file", "not_on_file"] | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        job_title: str | NotGiven = NOT_GIVEN,
        key_employee_status: Literal["key_employee", "non_key_employee"] | NotGiven = NOT_GIVEN,
        last_name: str | NotGiven = NOT_GIVEN,
        middle_name: str | NotGiven = NOT_GIVEN,
        military_status: Literal["active", "reserve"] | NotGiven = NOT_GIVEN,
        mobile: str | NotGiven = NOT_GIVEN,
        note: str | NotGiven = NOT_GIVEN,
        original_hire_date: Union[str, date] | NotGiven = NOT_GIVEN,
        overtime_exempt_status: Literal["exempt", "non_exempt"] | NotGiven = NOT_GIVEN,
        pager: str | NotGiven = NOT_GIVEN,
        pager_pin: str | NotGiven = NOT_GIVEN,
        phone: str | NotGiven = NOT_GIVEN,
        print_as: str | NotGiven = NOT_GIVEN,
        salutation: str | NotGiven = NOT_GIVEN,
        ssn: str | NotGiven = NOT_GIVEN,
        supervisor_id: str | NotGiven = NOT_GIVEN,
        target_bonus: str | NotGiven = NOT_GIVEN,
        termination_date: Union[str, date] | NotGiven = NOT_GIVEN,
        us_citizenship_status: Literal["citizen", "non_citizen"] | NotGiven = NOT_GIVEN,
        us_veteran_status: Literal["veteran", "non_veteran"] | NotGiven = NOT_GIVEN,
        work_authorization_expiration_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Employee:
        """
        Creates a new employee.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_number: The employee's account number, which appears in the QuickBooks chart of
              accounts, reports, and graphs.

              Note that if the "Use Account Numbers" preference is turned off in QuickBooks,
              the account number may not be visible in the user interface, but it can still be
              set and retrieved through the API.

          additional_notes: Additional notes about this employee.

          address: The employee's address.

              If the company uses QuickBooks Payroll for this employee, this address must
              specify a complete address, including city, state, ZIP (or postal) code, and at
              least one line of the street address.

          adjusted_service_date: The adjusted service date for this employee, in ISO 8601 format (YYYY-MM-DD).
              This date accounts for previous employment periods or leaves that affect
              seniority.

          alternate_phone: The employee's alternate telephone number.

          billing_rate_id: The employee's billing rate, used to override service item rates in time
              tracking activities.

          birth_date: This employee's date of birth, in ISO 8601 format (YYYY-MM-DD).

          custom_contact_fields: Additional custom contact fields for this employee, such as phone numbers or
              email addresses.

          department: The employee's department. Found in the "employment job details" section of the
              employee's record in QuickBooks.

          description: A description of this employee. Found in the "employment job details" section of
              the employee's record in QuickBooks.

          disability_description: A description of this employee's disability.

          disability_status: Indicates whether this employee is disabled.

          email: The employee's email address.

          emergency_contact: The employee's emergency contacts.

          employee_payroll: The employee's payroll information.

          employee_type: The employee type. This affects payroll taxes - a statutory employee is defined
              as an employee by statute. Note that owners/partners are typically on the "Other
              Names" list in QuickBooks, but if listed as an employee their type will be
              `owner`.

          employment_status: Indicates whether this employee is a part-time or full-time employee.

          ethnicity: This employee's ethnicity.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          fax: The employee's fax number.

          first_name: The employee's first name.

              Maximum length: 25 characters.

          gender: This employee's gender.

          hired_date: The date this employee was hired, in ISO 8601 format (YYYY-MM-DD).

          i9_on_file_status: Indicates whether this employee's I-9 is on file.

          is_active: Indicates whether this employee is active. Inactive objects are typically hidden
              from views and reports in QuickBooks. Defaults to `true`.

          job_title: The employee's job title.

          key_employee_status: Indicates whether this employee is a key employee.

          last_name: The employee's last name.

              Maximum length: 25 characters.

          middle_name: The employee's middle name.

              Maximum length: 5 characters.

          military_status: This employee's military status if they are a U.S. veteran.

          mobile: The employee's mobile phone number.

          note: A note or comment about this employee.

          original_hire_date: The original hire date for this employee, in ISO 8601 format (YYYY-MM-DD).

          overtime_exempt_status: Indicates whether this employee is exempt from overtime pay. This classification
              is based on U.S. labor laws (FLSA).

          pager: The employee's pager number.

          pager_pin: The employee's pager PIN.

          phone: The employee's primary telephone number.

          print_as: The name to use when printing this employee from QuickBooks. By default, this is
              the same as the `name` field.

          salutation: The employee's formal salutation title that precedes their name, such as "Mr.",
              "Ms.", or "Dr.".

          ssn: The employee's Social Security Number. The value can be with or without dashes.

              **NOTE**: This field cannot be changed after the employee is created.

          supervisor_id: The employee's supervisor. Found in the "employment job details" section of the
              employee's record in QuickBooks.

          target_bonus: The target bonus for this employee, represented as a decimal string. Found in
              the "employment job details" section of the employee's record in QuickBooks.

          termination_date: The date this employee's employment ended with the company, in ISO 8601 format
              (YYYY-MM-DD). This is also known as the released date or separation date.

          us_citizenship_status: Indicates whether this employee is a U.S. citizen.

          us_veteran_status: Indicates whether this employee is a U.S. veteran.

          work_authorization_expiration_date: The date this employee's work authorization expires, in ISO 8601 format
              (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/employees",
            body=await async_maybe_transform(
                {
                    "account_number": account_number,
                    "additional_notes": additional_notes,
                    "address": address,
                    "adjusted_service_date": adjusted_service_date,
                    "alternate_phone": alternate_phone,
                    "billing_rate_id": billing_rate_id,
                    "birth_date": birth_date,
                    "custom_contact_fields": custom_contact_fields,
                    "department": department,
                    "description": description,
                    "disability_description": disability_description,
                    "disability_status": disability_status,
                    "email": email,
                    "emergency_contact": emergency_contact,
                    "employee_payroll": employee_payroll,
                    "employee_type": employee_type,
                    "employment_status": employment_status,
                    "ethnicity": ethnicity,
                    "external_id": external_id,
                    "fax": fax,
                    "first_name": first_name,
                    "gender": gender,
                    "hired_date": hired_date,
                    "i9_on_file_status": i9_on_file_status,
                    "is_active": is_active,
                    "job_title": job_title,
                    "key_employee_status": key_employee_status,
                    "last_name": last_name,
                    "middle_name": middle_name,
                    "military_status": military_status,
                    "mobile": mobile,
                    "note": note,
                    "original_hire_date": original_hire_date,
                    "overtime_exempt_status": overtime_exempt_status,
                    "pager": pager,
                    "pager_pin": pager_pin,
                    "phone": phone,
                    "print_as": print_as,
                    "salutation": salutation,
                    "ssn": ssn,
                    "supervisor_id": supervisor_id,
                    "target_bonus": target_bonus,
                    "termination_date": termination_date,
                    "us_citizenship_status": us_citizenship_status,
                    "us_veteran_status": us_veteran_status,
                    "work_authorization_expiration_date": work_authorization_expiration_date,
                },
                employee_create_params.EmployeeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Employee,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Employee:
        """
        Retrieves an employee by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the employee to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/employees/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Employee,
        )

    async def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        account_number: str | NotGiven = NOT_GIVEN,
        additional_notes: Iterable[employee_update_params.AdditionalNote] | NotGiven = NOT_GIVEN,
        address: employee_update_params.Address | NotGiven = NOT_GIVEN,
        adjusted_service_date: Union[str, date] | NotGiven = NOT_GIVEN,
        alternate_phone: str | NotGiven = NOT_GIVEN,
        billing_rate_id: str | NotGiven = NOT_GIVEN,
        birth_date: Union[str, date] | NotGiven = NOT_GIVEN,
        custom_contact_fields: Iterable[employee_update_params.CustomContactField] | NotGiven = NOT_GIVEN,
        department: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        disability_description: str | NotGiven = NOT_GIVEN,
        disability_status: Literal["disabled", "non_disabled"] | NotGiven = NOT_GIVEN,
        email: str | NotGiven = NOT_GIVEN,
        emergency_contact: employee_update_params.EmergencyContact | NotGiven = NOT_GIVEN,
        employee_payroll: employee_update_params.EmployeePayroll | NotGiven = NOT_GIVEN,
        employee_type: Literal["officer", "owner", "regular", "statutory"] | NotGiven = NOT_GIVEN,
        employment_status: Literal["full_time", "part_time"] | NotGiven = NOT_GIVEN,
        ethnicity: Literal["american_indian", "asian", "black", "hawaiian", "hispanic", "white", "two_or_more_races"]
        | NotGiven = NOT_GIVEN,
        fax: str | NotGiven = NOT_GIVEN,
        first_name: str | NotGiven = NOT_GIVEN,
        hired_date: Union[str, date] | NotGiven = NOT_GIVEN,
        i9_on_file_status: Literal["on_file", "not_on_file"] | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        job_title: str | NotGiven = NOT_GIVEN,
        key_employee_status: Literal["key_employee", "non_key_employee"] | NotGiven = NOT_GIVEN,
        last_name: str | NotGiven = NOT_GIVEN,
        middle_name: str | NotGiven = NOT_GIVEN,
        military_status: Literal["active", "reserve"] | NotGiven = NOT_GIVEN,
        mobile: str | NotGiven = NOT_GIVEN,
        note: str | NotGiven = NOT_GIVEN,
        original_hire_date: Union[str, date] | NotGiven = NOT_GIVEN,
        overtime_exempt_status: Literal["exempt", "non_exempt"] | NotGiven = NOT_GIVEN,
        pager: str | NotGiven = NOT_GIVEN,
        pager_pin: str | NotGiven = NOT_GIVEN,
        phone: str | NotGiven = NOT_GIVEN,
        print_as: str | NotGiven = NOT_GIVEN,
        salutation: str | NotGiven = NOT_GIVEN,
        supervisor_id: str | NotGiven = NOT_GIVEN,
        target_bonus: str | NotGiven = NOT_GIVEN,
        termination_date: Union[str, date] | NotGiven = NOT_GIVEN,
        us_citizenship_status: Literal["citizen", "non_citizen"] | NotGiven = NOT_GIVEN,
        us_veteran_status: Literal["veteran", "non_veteran"] | NotGiven = NOT_GIVEN,
        work_authorization_expiration_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Employee:
        """
        Updates an existing employee.

        Args:
          id: The QuickBooks-assigned unique identifier of the employee to update.

          revision_number: The current QuickBooks-assigned revision number of the employee object you are
              updating, which you can get by fetching the object first. Provide the most
              recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_number: The employee's account number, which appears in the QuickBooks chart of
              accounts, reports, and graphs.

              Note that if the "Use Account Numbers" preference is turned off in QuickBooks,
              the account number may not be visible in the user interface, but it can still be
              set and retrieved through the API.

          additional_notes: Additional notes about this employee.

          address: The employee's address.

              If the company uses QuickBooks Payroll for this employee, this address must
              specify a complete address, including city, state, ZIP (or postal) code, and at
              least one line of the street address.

          adjusted_service_date: The adjusted service date for this employee, in ISO 8601 format (YYYY-MM-DD).
              This date accounts for previous employment periods or leaves that affect
              seniority.

          alternate_phone: The employee's alternate telephone number.

          billing_rate_id: The employee's billing rate, used to override service item rates in time
              tracking activities.

          birth_date: This employee's date of birth, in ISO 8601 format (YYYY-MM-DD).

          custom_contact_fields: Additional custom contact fields for this employee, such as phone numbers or
              email addresses.

          department: The employee's department. Found in the "employment job details" section of the
              employee's record in QuickBooks.

          description: A description of this employee. Found in the "employment job details" section of
              the employee's record in QuickBooks.

          disability_description: A description of this employee's disability.

          disability_status: Indicates whether this employee is disabled.

          email: The employee's email address.

          emergency_contact: The employee's emergency contacts.

          employee_payroll: The employee's payroll information.

          employee_type: The employee type. This affects payroll taxes - a statutory employee is defined
              as an employee by statute. Note that owners/partners are typically on the "Other
              Names" list in QuickBooks, but if listed as an employee their type will be
              `owner`.

          employment_status: Indicates whether this employee is a part-time or full-time employee.

          ethnicity: This employee's ethnicity.

          fax: The employee's fax number.

          first_name: The employee's first name.

              Maximum length: 25 characters.

          hired_date: The date this employee was hired, in ISO 8601 format (YYYY-MM-DD).

          i9_on_file_status: Indicates whether this employee's I-9 is on file.

          is_active: Indicates whether this employee is active. Inactive objects are typically hidden
              from views and reports in QuickBooks. Defaults to `true`.

          job_title: The employee's job title.

          key_employee_status: Indicates whether this employee is a key employee.

          last_name: The employee's last name.

              Maximum length: 25 characters.

          middle_name: The employee's middle name.

              Maximum length: 5 characters.

          military_status: This employee's military status if they are a U.S. veteran.

          mobile: The employee's mobile phone number.

          note: A note or comment about this employee.

          original_hire_date: The original hire date for this employee, in ISO 8601 format (YYYY-MM-DD).

          overtime_exempt_status: Indicates whether this employee is exempt from overtime pay. This classification
              is based on U.S. labor laws (FLSA).

          pager: The employee's pager number.

          pager_pin: The employee's pager PIN.

          phone: The employee's primary telephone number.

          print_as: The name to use when printing this employee from QuickBooks. By default, this is
              the same as the `name` field.

          salutation: The employee's formal salutation title that precedes their name, such as "Mr.",
              "Ms.", or "Dr.".

          supervisor_id: The employee's supervisor. Found in the "employment job details" section of the
              employee's record in QuickBooks.

          target_bonus: The target bonus for this employee, represented as a decimal string. Found in
              the "employment job details" section of the employee's record in QuickBooks.

          termination_date: The date this employee's employment ended with the company, in ISO 8601 format
              (YYYY-MM-DD). This is also known as the released date or separation date.

          us_citizenship_status: Indicates whether this employee is a U.S. citizen.

          us_veteran_status: Indicates whether this employee is a U.S. veteran.

          work_authorization_expiration_date: The date this employee's work authorization expires, in ISO 8601 format
              (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/employees/{id}",
            body=await async_maybe_transform(
                {
                    "revision_number": revision_number,
                    "account_number": account_number,
                    "additional_notes": additional_notes,
                    "address": address,
                    "adjusted_service_date": adjusted_service_date,
                    "alternate_phone": alternate_phone,
                    "billing_rate_id": billing_rate_id,
                    "birth_date": birth_date,
                    "custom_contact_fields": custom_contact_fields,
                    "department": department,
                    "description": description,
                    "disability_description": disability_description,
                    "disability_status": disability_status,
                    "email": email,
                    "emergency_contact": emergency_contact,
                    "employee_payroll": employee_payroll,
                    "employee_type": employee_type,
                    "employment_status": employment_status,
                    "ethnicity": ethnicity,
                    "fax": fax,
                    "first_name": first_name,
                    "hired_date": hired_date,
                    "i9_on_file_status": i9_on_file_status,
                    "is_active": is_active,
                    "job_title": job_title,
                    "key_employee_status": key_employee_status,
                    "last_name": last_name,
                    "middle_name": middle_name,
                    "military_status": military_status,
                    "mobile": mobile,
                    "note": note,
                    "original_hire_date": original_hire_date,
                    "overtime_exempt_status": overtime_exempt_status,
                    "pager": pager,
                    "pager_pin": pager_pin,
                    "phone": phone,
                    "print_as": print_as,
                    "salutation": salutation,
                    "supervisor_id": supervisor_id,
                    "target_bonus": target_bonus,
                    "termination_date": termination_date,
                    "us_citizenship_status": us_citizenship_status,
                    "us_veteran_status": us_veteran_status,
                    "work_authorization_expiration_date": work_authorization_expiration_date,
                },
                employee_update_params.EmployeeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Employee,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        names: List[str] | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Employee, AsyncCursorPage[Employee]]:
        """Returns a list of employees.

        Use the `cursor` parameter to paginate through the
        results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          ids: Filter for specific employees by their QuickBooks-assigned unique identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          name_contains: Filter for employees whose `name` contains this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameStartsWith` or
              `nameEndsWith`.

          name_ends_with: Filter for employees whose `name` ends with this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameContains` or
              `nameStartsWith`.

          name_from: Filter for employees whose `name` is alphabetically greater than or equal to
              this value.

          names: Filter for specific employees by their name(s), case-insensitive. Like `id`,
              `name` is a unique identifier for an employee.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          name_starts_with: Filter for employees whose `name` starts with this substring, case-insensitive.
              NOTE: If you use this parameter, you cannot also use `nameContains` or
              `nameEndsWith`.

          name_to: Filter for employees whose `name` is alphabetically less than or equal to this
              value.

          status: Filter for employees that are active, inactive, or both.

          updated_after: Filter for employees updated on or after this date and time, in ISO 8601 format
              (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
              assumed to be 00:00:00 of that day.

          updated_before: Filter for employees updated on or before this date and time, in ISO 8601 format
              (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
              assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/employees",
            page=AsyncCursorPage[Employee],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "names": names,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    employee_list_params.EmployeeListParams,
                ),
            ),
            model=Employee,
        )


class EmployeesResourceWithRawResponse:
    def __init__(self, employees: EmployeesResource) -> None:
        self._employees = employees

        self.create = to_raw_response_wrapper(
            employees.create,
        )
        self.retrieve = to_raw_response_wrapper(
            employees.retrieve,
        )
        self.update = to_raw_response_wrapper(
            employees.update,
        )
        self.list = to_raw_response_wrapper(
            employees.list,
        )


class AsyncEmployeesResourceWithRawResponse:
    def __init__(self, employees: AsyncEmployeesResource) -> None:
        self._employees = employees

        self.create = async_to_raw_response_wrapper(
            employees.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            employees.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            employees.update,
        )
        self.list = async_to_raw_response_wrapper(
            employees.list,
        )


class EmployeesResourceWithStreamingResponse:
    def __init__(self, employees: EmployeesResource) -> None:
        self._employees = employees

        self.create = to_streamed_response_wrapper(
            employees.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            employees.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            employees.update,
        )
        self.list = to_streamed_response_wrapper(
            employees.list,
        )


class AsyncEmployeesResourceWithStreamingResponse:
    def __init__(self, employees: AsyncEmployeesResource) -> None:
        self._employees = employees

        self.create = async_to_streamed_response_wrapper(
            employees.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            employees.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            employees.update,
        )
        self.list = async_to_streamed_response_wrapper(
            employees.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from conductor import Conductor, AsyncConductor
from tests.utils import assert_matches_type
from conductor._utils import parse_date
from conductor.types.qbd import Transaction
from conductor.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTransactions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Conductor) -> None:
        transaction = client.qbd.transactions.retrieve(
            id="123ABC-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(Transaction, transaction, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Conductor) -> None:
        response = client.qbd.transactions.with_raw_response.retrieve(
            id="123ABC-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(Transaction, transaction, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Conductor) -> None:
        with client.qbd.transactions.with_streaming_response.retrieve(
            id="123ABC-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(Transaction, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Conductor) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.qbd.transactions.with_raw_response.retrieve(
                id="",
                conductor_end_user_id="end_usr_1234567abcdefg",
            )

    @parametrize
    def test_method_list(self, client: Conductor) -> None:
        transaction = client.qbd.transactions.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(SyncCursorPage[Transaction], transaction, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Conductor) -> None:
        transaction = client.qbd.transactions.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
            account_ids=["80000001-1234567890"],
            class_ids=["80000001-1234567890"],
            currency_ids=["80000001-1234567890"],
            cursor="12345678-abcd-abcd-example-1234567890ab",
            detail_level="all",
            entity_ids=["80000001-1234567890"],
            ids=["123ABC-1234567890"],
            item_ids=["80000001-1234567890"],
            limit=150,
            payment_status="closed",
            posting_status="either",
            ref_number_contains="INV-1234",
            ref_number_ends_with="1234",
            ref_number_from="INV-0001",
            ref_numbers=["TRANSACTION-1234"],
            ref_number_starts_with="INV",
            ref_number_to="INV-9999",
            transaction_date_from=parse_date("2021-01-01"),
            transaction_date_to=parse_date("2021-02-01"),
            transaction_type=["all"],
            updated_after="2021-01-01T12:34:56",
            updated_before="2021-02-01T12:34:56",
        )
        assert_matches_type(SyncCursorPage[Transaction], transaction, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Conductor) -> None:
        response = client.qbd.transactions.with_raw_response.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = response.parse()
        assert_matches_type(SyncCursorPage[Transaction], transaction, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Conductor) -> None:
        with client.qbd.transactions.with_streaming_response.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = response.parse()
            assert_matches_type(SyncCursorPage[Transaction], transaction, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTransactions:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncConductor) -> None:
        transaction = await async_client.qbd.transactions.retrieve(
            id="123ABC-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(Transaction, transaction, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncConductor) -> None:
        response = await async_client.qbd.transactions.with_raw_response.retrieve(
            id="123ABC-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(Transaction, transaction, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncConductor) -> None:
        async with async_client.qbd.transactions.with_streaming_response.retrieve(
            id="123ABC-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(Transaction, transaction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncConductor) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.qbd.transactions.with_raw_response.retrieve(
                id="",
                conductor_end_user_id="end_usr_1234567abcdefg",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncConductor) -> None:
        transaction = await async_client.qbd.transactions.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(AsyncCursorPage[Transaction], transaction, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncConductor) -> None:
        transaction = await async_client.qbd.transactions.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
            account_ids=["80000001-1234567890"],
            class_ids=["80000001-1234567890"],
            currency_ids=["80000001-1234567890"],
            cursor="12345678-abcd-abcd-example-1234567890ab",
            detail_level="all",
            entity_ids=["80000001-1234567890"],
            ids=["123ABC-1234567890"],
            item_ids=["80000001-1234567890"],
            limit=150,
            payment_status="closed",
            posting_status="either",
            ref_number_contains="INV-1234",
            ref_number_ends_with="1234",
            ref_number_from="INV-0001",
            ref_numbers=["TRANSACTION-1234"],
            ref_number_starts_with="INV",
            ref_number_to="INV-9999",
            transaction_date_from=parse_date("2021-01-01"),
            transaction_date_to=parse_date("2021-02-01"),
            transaction_type=["all"],
            updated_after="2021-01-01T12:34:56",
            updated_before="2021-02-01T12:34:56",
        )
        assert_matches_type(AsyncCursorPage[Transaction], transaction, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncConductor) -> None:
        response = await async_client.qbd.transactions.with_raw_response.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        transaction = await response.parse()
        assert_matches_type(AsyncCursorPage[Transaction], transaction, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncConductor) -> None:
        async with async_client.qbd.transactions.with_streaming_response.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            transaction = await response.parse()
            assert_matches_type(AsyncCursorPage[Transaction], transaction, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "Company",
    "AccountantCopy",
    "Address",
    "AddressForCustomer",
    "CustomField",
    "LegalAddress",
    "SubscribedServices",
    "SubscribedServicesService",
]


class AccountantCopy(BaseModel):
    accountant_copy_exists: bool = FieldInfo(alias="accountantCopyExists")
    """Indicates whether an accountant copy has been made for this company file.

    An accountant copy allows an accountant to work on the books while the business
    continues daily operations.
    """

    dividing_date: Optional[str] = FieldInfo(alias="dividingDate", default=None)
    """
    The fiscal period dividing date for accountant work, in ISO 8601 format
    (YYYY-MM-DD). While an accountant copy exists, transactions within this period
    cannot be modified or created. New accounts can be added, but existing accounts
    cannot have new subaccounts, be edited, merged, or made inactive. List items
    cannot be deleted or merged.
    """


class Address(BaseModel):
    city: Optional[str] = None
    """The city, district, suburb, town, or village name of the address."""

    country: Optional[str] = None
    """The country name of the address."""

    line1: Optional[str] = None
    """The first line of the address (e.g., street, PO Box, or company name)."""

    line2: Optional[str] = None
    """
    The second line of the address, if needed (e.g., apartment, suite, unit, or
    building).
    """

    line3: Optional[str] = None
    """The third line of the address, if needed."""

    line4: Optional[str] = None
    """The fourth line of the address, if needed."""

    line5: Optional[str] = None
    """The fifth line of the address, if needed."""

    note: Optional[str] = None
    """
    A note written at the bottom of the address in the form in which it appears,
    such as the invoice form.
    """

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """The postal code or ZIP code of the address."""

    state: Optional[str] = None
    """The state, county, province, or region name of the address."""


class AddressForCustomer(BaseModel):
    city: Optional[str] = None
    """The city, district, suburb, town, or village name of the address."""

    country: Optional[str] = None
    """The country name of the address."""

    line1: Optional[str] = None
    """The first line of the address (e.g., street, PO Box, or company name)."""

    line2: Optional[str] = None
    """
    The second line of the address, if needed (e.g., apartment, suite, unit, or
    building).
    """

    line3: Optional[str] = None
    """The third line of the address, if needed."""

    line4: Optional[str] = None
    """The fourth line of the address, if needed."""

    line5: Optional[str] = None
    """The fifth line of the address, if needed."""

    note: Optional[str] = None
    """
    A note written at the bottom of the address in the form in which it appears,
    such as the invoice form.
    """

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """The postal code or ZIP code of the address."""

    state: Optional[str] = None
    """The state, county, province, or region name of the address."""


class CustomField(BaseModel):
    name: str
    """The name of the custom field, unique for the specified `ownerId`.

    For public custom fields, this name is visible as a label in the QuickBooks UI.
    """

    owner_id: str = FieldInfo(alias="ownerId")
    """
    The identifier of the owner of the custom field, which QuickBooks internally
    calls a "data extension". For public custom fields visible in the UI, such as
    those added by the QuickBooks user, this is always "0". For private custom
    fields that are only visible to the application that created them, this is a
    valid GUID identifying the owning application. Internally, Conductor always
    fetches all public custom fields (those with an `ownerId` of "0") for all
    objects.
    """

    type: Literal[
        "amount_type",
        "date_time_type",
        "integer_type",
        "percent_type",
        "price_type",
        "quantity_type",
        "string_1024_type",
        "string_255_type",
    ]
    """The data type of this custom field."""

    value: str
    """The value of this custom field.

    The maximum length depends on the field's data type.
    """


class LegalAddress(BaseModel):
    city: Optional[str] = None
    """The city, district, suburb, town, or village name of the address."""

    country: Optional[str] = None
    """The country name of the address."""

    line1: Optional[str] = None
    """The first line of the address (e.g., street, PO Box, or company name)."""

    line2: Optional[str] = None
    """
    The second line of the address, if needed (e.g., apartment, suite, unit, or
    building).
    """

    line3: Optional[str] = None
    """The third line of the address, if needed."""

    line4: Optional[str] = None
    """The fourth line of the address, if needed."""

    line5: Optional[str] = None
    """The fifth line of the address, if needed."""

    note: Optional[str] = None
    """
    A note written at the bottom of the address in the form in which it appears,
    such as the invoice form.
    """

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """The postal code or ZIP code of the address."""

    state: Optional[str] = None
    """The state, county, province, or region name of the address."""


class SubscribedServicesService(BaseModel):
    domain: Optional[str] = None
    """The domain of this subscribed service"""

    name: str
    """The case-insensitive unique name of this service, unique across all services.

    **NOTE**: Services do not have a `fullName` field because they are not
    hierarchical objects, which is why `name` is unique for them but not for objects
    that have parents.
    """

    service_status: Optional[Literal["active", "expired", "never", "pending", "suspended", "terminated", "trial"]] = (
        FieldInfo(alias="serviceStatus", default=None)
    )
    """The status of this service's subscription."""


class SubscribedServices(BaseModel):
    services: List[SubscribedServicesService]
    """
    The list of Intuit services that this company is or has been subscribed to, for
    example, Intuit Payroll, QBMS.
    """


class Company(BaseModel):
    accountant_copy: Optional[AccountantCopy] = FieldInfo(alias="accountantCopy", default=None)
    """Information about the accountant's copy for this company file.

    An accountant's copy allows an accountant to make changes while the business
    continues normal operations. It includes a dividing date that defines the fiscal
    period the accountant can work on, with restrictions on transactions and
    accounts within that period. While an accountant copy exists, users cannot
    modify transactions dated on or before the dividing date, cannot add subaccounts
    to existing accounts, and cannot edit, merge, or make existing accounts
    inactive.
    """

    address: Optional[Address] = None
    """
    The company's address, used on its invoices, checks, and other forms (along with
    `companyName`). This is different from the company's legal address used on tax
    forms and pay stubs (along with `legalCompanyName`).
    """

    address_for_customer: Optional[AddressForCustomer] = FieldInfo(alias="addressForCustomer", default=None)
    """The address where this company receives mail from its customers."""

    company_name: Optional[str] = FieldInfo(alias="companyName", default=None)
    """The name of the QuickBooks user's business associated with this company.

    This name is used on invoices, checks, and other forms, while `legalCompanyName`
    is used on tax forms and pay stubs.
    """

    company_type: Optional[str] = FieldInfo(alias="companyType", default=None)
    """
    The company type, which the QuickBooks user selected from a list when creating
    the company file.
    """

    custom_fields: List[CustomField] = FieldInfo(alias="customFields")
    """
    The custom fields for the company object, added as user-defined data extensions,
    not included in the standard QuickBooks object.
    """

    ein: Optional[str] = None
    """The company's Employer Identification Number."""

    email: Optional[str] = None
    """The company's email address."""

    fax: Optional[str] = None
    """The company's fax number."""

    fiscal_year_start_month: Optional[
        Literal[
            "january",
            "february",
            "march",
            "april",
            "may",
            "june",
            "july",
            "august",
            "september",
            "october",
            "november",
            "december",
        ]
    ] = FieldInfo(alias="fiscalYearStartMonth", default=None)
    """
    The first month of this company's fiscal year, which determines the default date
    range for financial reports.
    """

    income_tax_year_start_month: Optional[
        Literal[
            "january",
            "february",
            "march",
            "april",
            "may",
            "june",
            "july",
            "august",
            "september",
            "october",
            "november",
            "december",
        ]
    ] = FieldInfo(alias="incomeTaxYearStartMonth", default=None)
    """
    The first month of this company's income tax year, which determines the default
    date range for financial reports.
    """

    is_sample_company_file: bool = FieldInfo(alias="isSampleCompanyFile")
    """
    Indicates whether the connected QuickBooks company file is a "sample file",
    which is a mock company file used for testing.
    """

    legal_address: Optional[LegalAddress] = FieldInfo(alias="legalAddress", default=None)
    """
    The company's legal address used on its tax forms and pay stubs (along with
    `legalCompanyName`). This is different from the company's `address` used on
    invoices, checks, and other forms (along with `companyName`).
    """

    legal_company_name: Optional[str] = FieldInfo(alias="legalCompanyName", default=None)
    """The legal name of this company's business, as specified in QuickBooks.

    This value is used on tax forms and pay stubs, while `companyName` is used on
    invoices, checks, and other forms.
    """

    phone: Optional[str] = None
    """The company's primary telephone number."""

    ssn: Optional[str] = None
    """The company's Social Security Number. The value can be with or without dashes.

    **NOTE**: This field cannot be changed after the company is created.
    """

    subscribed_services: Optional[SubscribedServices] = FieldInfo(alias="subscribedServices", default=None)
    """
    The Intuit services that this company is or has been subscribed to, such as
    Intuit Payroll.
    """

    tax_form: Optional[
        Literal[
            "form_1040", "form_1065", "form_1120", "form_1120s", "form_990", "form_990pf", "form_990t", "other_or_none"
        ]
    ] = FieldInfo(alias="taxForm", default=None)
    """The tax form that the QuickBooks user expects to file for this company's taxes.

    When a specific tax form is selected (any value other than `other_or_none`),
    QuickBooks allows associating each account with a specific tax form line. This
    association appears in account query responses.
    """

    website: Optional[str] = None
    """The company's public website."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["EndUserPassthroughParams"]


class EndUserPassthroughParams(TypedDict, total=False):
    id: Required[str]
    """The ID of the end-user who owns the integration connection."""

    qbd_payload: Required[Dict[str, object]]
    """The request body to send to the integration connection."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "Employee",
    "AdditionalNote",
    "Address",
    "BillingRate",
    "CustomContactField",
    "CustomField",
    "EmergencyContact",
    "EmergencyContactPrimaryContact",
    "EmergencyContactSecondaryContact",
    "EmployeePayroll",
    "EmployeePayrollClass",
    "EmployeePayrollEarning",
    "EmployeePayrollEarningPayrollWageItem",
    "EmployeePayrollSickHours",
    "EmployeePayrollVacationHours",
    "Supervisor",
]


class AdditionalNote(BaseModel):
    id: float
    """The auto-incrementing identifier assigned by QuickBooks to this note."""

    date: Optional[datetime.date] = None
    """The date this note was last updated, in ISO 8601 format (YYYY-MM-DD)."""

    note: str
    """The text of this note."""


class Address(BaseModel):
    city: Optional[str] = None
    """The city, district, suburb, town, or village name of the employee address."""

    country: Optional[str] = None
    """The country name of the employee address."""

    line1: Optional[str] = None
    """The first line of the employee address (e.g., street, PO Box, or company name)."""

    line2: Optional[str] = None
    """
    The second line of the employee address, if needed (e.g., apartment, suite,
    unit, or building).
    """

    line3: Optional[str] = None
    """The third line of the employee address, if needed."""

    line4: Optional[str] = None
    """The fourth line of the employee address, if needed."""

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """The postal code or ZIP code of the employee address."""

    state: Optional[
        Literal[
            "none",
            "armed_forces_americas",
            "armed_forces_europe",
            "armed_forces_pacific",
            "AK",
            "AL",
            "AR",
            "AZ",
            "CA",
            "CO",
            "CT",
            "DC",
            "DE",
            "FL",
            "GA",
            "HI",
            "IA",
            "ID",
            "IL",
            "IN",
            "KS",
            "KY",
            "LA",
            "MA",
            "MD",
            "ME",
            "MI",
            "MN",
            "MO",
            "MS",
            "MT",
            "NB",
            "NC",
            "ND",
            "NE",
            "NH",
            "NJ",
            "NM",
            "NV",
            "NY",
            "OH",
            "OK",
            "OR",
            "PA",
            "PR",
            "RI",
            "SC",
            "SD",
            "TN",
            "TX",
            "UT",
            "VA",
            "VT",
            "WA",
            "WI",
            "WV",
            "WY",
        ]
    ] = None
    """The U.S.

    state of the employee address. QuickBooks requires this field to be a U.S. state
    abbreviation (e.g., "CA" for California). See enum for all possible values.
    """


class BillingRate(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class CustomContactField(BaseModel):
    name: str
    """The name of the contact field (e.g., "old address", "secondary phone")."""

    value: str
    """The value of the contact field."""


class CustomField(BaseModel):
    name: str
    """The name of the custom field, unique for the specified `ownerId`.

    For public custom fields, this name is visible as a label in the QuickBooks UI.
    """

    owner_id: str = FieldInfo(alias="ownerId")
    """
    The identifier of the owner of the custom field, which QuickBooks internally
    calls a "data extension". For public custom fields visible in the UI, such as
    those added by the QuickBooks user, this is always "0". For private custom
    fields that are only visible to the application that created them, this is a
    valid GUID identifying the owning application. Internally, Conductor always
    fetches all public custom fields (those with an `ownerId` of "0") for all
    objects.
    """

    type: Literal[
        "amount_type",
        "date_time_type",
        "integer_type",
        "percent_type",
        "price_type",
        "quantity_type",
        "string_1024_type",
        "string_255_type",
    ]
    """The data type of this custom field."""

    value: str
    """The value of this custom field.

    The maximum length depends on the field's data type.
    """


class EmergencyContactPrimaryContact(BaseModel):
    name: str
    """The name of the contact field (e.g., "old address", "secondary phone")."""

    relation: Optional[
        Literal["brother", "daughter", "father", "friend", "mother", "other", "partner", "sister", "son", "spouse"]
    ] = None
    """The relationship of the employee to the employee."""

    value: str
    """The value of the contact field."""


class EmergencyContactSecondaryContact(BaseModel):
    name: str
    """The name of the contact field (e.g., "old address", "secondary phone")."""

    relation: Optional[
        Literal["brother", "daughter", "father", "friend", "mother", "other", "partner", "sister", "son", "spouse"]
    ] = None
    """The relationship of the employee to the employee."""

    value: str
    """The value of the contact field."""


class EmergencyContact(BaseModel):
    primary_contact: Optional[EmergencyContactPrimaryContact] = FieldInfo(alias="primaryContact", default=None)
    """The employee's primary emergency contact."""

    secondary_contact: Optional[EmergencyContactSecondaryContact] = FieldInfo(alias="secondaryContact", default=None)
    """The employee's secondary emergency contact."""


class EmployeePayrollClass(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class EmployeePayrollEarningPayrollWageItem(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class EmployeePayrollEarning(BaseModel):
    payroll_wage_item: EmployeePayrollEarningPayrollWageItem = FieldInfo(alias="payrollWageItem")
    """
    The payroll wage item that defines how this employee is paid (e.g., Regular Pay,
    Overtime Pay). This determines the payment scheme used for payroll calculations.
    """

    rate: Optional[str] = None
    """The hourly rate for this employee, represented as a decimal string."""

    rate_percent: Optional[str] = FieldInfo(alias="ratePercent", default=None)
    """The hourly rate for this employee expressed as a percentage."""


class EmployeePayrollSickHours(BaseModel):
    accrual_period: Optional[Literal["accrues_annually", "accrues_hourly", "accrues_per_paycheck"]] = FieldInfo(
        alias="accrualPeriod", default=None
    )
    """How frequently the employee's sick hours are accrued."""

    accrual_start_date: Optional[datetime.date] = FieldInfo(alias="accrualStartDate", default=None)
    """
    The date the employee's sick hours began to accrue, in ISO 8601 format
    (YYYY-MM-DD).
    """

    hours_accrued_per_period: Optional[str] = FieldInfo(alias="hoursAccruedPerPeriod", default=None)
    """
    The number of sick hours the employee will accrue per accrual period, in ISO
    8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is
    represented as PT1H30M.
    """

    hours_available: Optional[str] = FieldInfo(alias="hoursAvailable", default=None)
    """
    The total number of sick hours currently available for the employee to use, in
    ISO 8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30
    minutes is represented as PT1H30M. Defaults to 0.
    """

    hours_used: Optional[str] = FieldInfo(alias="hoursUsed", default=None)
    """
    The number of sick hours the employee has used, in ISO 8601 format for time
    intervals (PTnHnMnS). For example, 1 hour and 30 minutes is represented as
    PT1H30M.
    """

    maximum_hours: Optional[str] = FieldInfo(alias="maximumHours", default=None)
    """
    The maximum number of sick hours the employee can accrue, in ISO 8601 format for
    time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is represented as
    PT1H30M.
    """

    resets_hours_each_year: Optional[bool] = FieldInfo(alias="resetsHoursEachYear", default=None)
    """
    Indicates whether the employee's sick hours reset to zero at the beginning of
    the new year.
    """


class EmployeePayrollVacationHours(BaseModel):
    accrual_period: Optional[Literal["accrues_annually", "accrues_hourly", "accrues_per_paycheck"]] = FieldInfo(
        alias="accrualPeriod", default=None
    )
    """How frequently the employee's vacation hours are accrued."""

    accrual_start_date: Optional[datetime.date] = FieldInfo(alias="accrualStartDate", default=None)
    """
    The date the employee's vacation hours began to accrue, in ISO 8601 format
    (YYYY-MM-DD).
    """

    hours_accrued_per_period: Optional[str] = FieldInfo(alias="hoursAccruedPerPeriod", default=None)
    """
    The number of vacation hours the employee will accrue per accrual period, in ISO
    8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is
    represented as PT1H30M.
    """

    hours_available: Optional[str] = FieldInfo(alias="hoursAvailable", default=None)
    """
    The total number of vacation hours currently available for the employee to use,
    in ISO 8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30
    minutes is represented as PT1H30M. Defaults to 0.
    """

    hours_used: Optional[str] = FieldInfo(alias="hoursUsed", default=None)
    """
    The number of vacation hours the employee has used, in ISO 8601 format for time
    intervals (PTnHnMnS). For example, 1 hour and 30 minutes is represented as
    PT1H30M.
    """

    maximum_hours: Optional[str] = FieldInfo(alias="maximumHours", default=None)
    """
    The maximum number of vacation hours the employee can accrue, in ISO 8601 format
    for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is represented
    as PT1H30M.
    """

    resets_hours_each_year: Optional[bool] = FieldInfo(alias="resetsHoursEachYear", default=None)
    """
    Indicates whether the employee's vacation hours reset to zero at the beginning
    of the new year.
    """


class EmployeePayroll(BaseModel):
    class_: Optional[EmployeePayrollClass] = FieldInfo(alias="class", default=None)
    """The employee's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default.
    """

    earnings: List[EmployeePayrollEarning]
    """The employee's earnings."""

    pay_period: Optional[Literal["biweekly", "daily", "monthly", "quarterly", "semimonthly", "weekly", "yearly"]] = (
        FieldInfo(alias="payPeriod", default=None)
    )
    """How frequently this employee is paid (e.g., weekly, biweekly, monthly).

    This determines the schedule for generating paychecks.
    """

    sick_hours: Optional[EmployeePayrollSickHours] = FieldInfo(alias="sickHours", default=None)
    """
    The employee's sick hours, including how sick time is accrued and the total
    hours accrued.
    """

    use_time_data_to_create_paychecks: Optional[Literal["does_not_use_time_data", "not_set", "uses_time_data"]] = (
        FieldInfo(alias="useTimeDataToCreatePaychecks", default=None)
    )
    """
    Indicates whether this employee is using time-tracking data to create paychecks.
    """

    vacation_hours: Optional[EmployeePayrollVacationHours] = FieldInfo(alias="vacationHours", default=None)
    """
    The employee's vacation hours, including how vacation time is accrued and the
    total hours accrued.
    """


class Supervisor(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class Employee(BaseModel):
    id: str
    """The unique identifier assigned by QuickBooks to this employee.

    This ID is unique across all employees but not across different QuickBooks
    object types.
    """

    account_number: Optional[str] = FieldInfo(alias="accountNumber", default=None)
    """
    The employee's account number, which appears in the QuickBooks chart of
    accounts, reports, and graphs.

    Note that if the "Use Account Numbers" preference is turned off in QuickBooks,
    the account number may not be visible in the user interface, but it can still be
    set and retrieved through the API.
    """

    additional_notes: List[AdditionalNote] = FieldInfo(alias="additionalNotes")
    """Additional notes about this employee."""

    address: Optional[Address] = None
    """The employee's address.

    If the company uses QuickBooks Payroll for this employee, this address must
    specify a complete address, including city, state, ZIP (or postal) code, and at
    least one line of the street address.
    """

    adjusted_service_date: Optional[datetime.date] = FieldInfo(alias="adjustedServiceDate", default=None)
    """The adjusted service date for this employee, in ISO 8601 format (YYYY-MM-DD).

    This date accounts for previous employment periods or leaves that affect
    seniority.
    """

    alternate_phone: Optional[str] = FieldInfo(alias="alternatePhone", default=None)
    """The employee's alternate telephone number."""

    billing_rate: Optional[BillingRate] = FieldInfo(alias="billingRate", default=None)
    """
    The employee's billing rate, used to override service item rates in time
    tracking activities.
    """

    birth_date: Optional[datetime.date] = FieldInfo(alias="birthDate", default=None)
    """This employee's date of birth, in ISO 8601 format (YYYY-MM-DD)."""

    created_at: str = FieldInfo(alias="createdAt")
    """
    The date and time when this employee was created, in ISO 8601 format
    (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time zone
    in QuickBooks.
    """

    custom_contact_fields: List[CustomContactField] = FieldInfo(alias="customContactFields")
    """
    Additional custom contact fields for this employee, such as phone numbers or
    email addresses.
    """

    custom_fields: List[CustomField] = FieldInfo(alias="customFields")
    """
    The custom fields for the employee object, added as user-defined data
    extensions, not included in the standard QuickBooks object.
    """

    department: Optional[str] = None
    """The employee's department.

    Found in the "employment job details" section of the employee's record in
    QuickBooks.
    """

    description: Optional[str] = None
    """A description of this employee.

    Found in the "employment job details" section of the employee's record in
    QuickBooks.
    """

    disability_description: Optional[str] = FieldInfo(alias="disabilityDescription", default=None)
    """A description of this employee's disability."""

    disability_status: Optional[Literal["disabled", "non_disabled"]] = FieldInfo(alias="disabilityStatus", default=None)
    """Indicates whether this employee is disabled."""

    email: Optional[str] = None
    """The employee's email address."""

    emergency_contact: Optional[EmergencyContact] = FieldInfo(alias="emergencyContact", default=None)
    """The employee's emergency contacts."""

    employee_payroll: Optional[EmployeePayroll] = FieldInfo(alias="employeePayroll", default=None)
    """The employee's payroll information."""

    employee_type: Optional[Literal["officer", "owner", "regular", "statutory"]] = FieldInfo(
        alias="employeeType", default=None
    )
    """The employee type.

    This affects payroll taxes - a statutory employee is defined as an employee by
    statute. Note that owners/partners are typically on the "Other Names" list in
    QuickBooks, but if listed as an employee their type will be `owner`.
    """

    employment_status: Optional[Literal["full_time", "part_time"]] = FieldInfo(alias="employmentStatus", default=None)
    """Indicates whether this employee is a part-time or full-time employee."""

    ethnicity: Optional[
        Literal["american_indian", "asian", "black", "hawaiian", "hispanic", "white", "two_or_more_races"]
    ] = None
    """This employee's ethnicity."""

    external_id: Optional[str] = FieldInfo(alias="externalId", default=None)
    """
    A globally unique identifier (GUID) you, the developer, can provide for tracking
    this object in your external system. This field is immutable and can only be set
    during object creation.
    """

    fax: Optional[str] = None
    """The employee's fax number."""

    first_name: Optional[str] = FieldInfo(alias="firstName", default=None)
    """The employee's first name."""

    gender: Optional[Literal["male", "female"]] = None
    """This employee's gender."""

    hired_date: Optional[datetime.date] = FieldInfo(alias="hiredDate", default=None)
    """The date this employee was hired, in ISO 8601 format (YYYY-MM-DD)."""

    i9_on_file_status: Optional[Literal["on_file", "not_on_file"]] = FieldInfo(alias="i9OnFileStatus", default=None)
    """Indicates whether this employee's I-9 is on file."""

    is_active: bool = FieldInfo(alias="isActive")
    """Indicates whether this employee is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """

    job_title: Optional[str] = FieldInfo(alias="jobTitle", default=None)
    """The employee's job title."""

    key_employee_status: Optional[Literal["key_employee", "non_key_employee"]] = FieldInfo(
        alias="keyEmployeeStatus", default=None
    )
    """Indicates whether this employee is a key employee."""

    last_name: Optional[str] = FieldInfo(alias="lastName", default=None)
    """The employee's last name."""

    middle_name: Optional[str] = FieldInfo(alias="middleName", default=None)
    """The employee's middle name."""

    military_status: Optional[Literal["active", "reserve"]] = FieldInfo(alias="militaryStatus", default=None)
    """This employee's military status if they are a U.S. veteran."""

    mobile: Optional[str] = None
    """The employee's mobile phone number."""

    name: str
    """The case-insensitive unique name of this employee, unique across all employees.

    A concatenation of the employee's `firstName`, `middleName`, and `lastName`
    fields.

    **NOTE**: Employees do not have a `fullName` field because they are not
    hierarchical objects, which is why `name` is unique for them but not for objects
    that have parents.
    """

    note: Optional[str] = None
    """A note or comment about this employee."""

    object_type: Literal["qbd_employee"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"qbd_employee"`."""

    original_hire_date: Optional[datetime.date] = FieldInfo(alias="originalHireDate", default=None)
    """The original hire date for this employee, in ISO 8601 format (YYYY-MM-DD)."""

    overtime_exempt_status: Optional[Literal["exempt", "non_exempt"]] = FieldInfo(
        alias="overtimeExemptStatus", default=None
    )
    """Indicates whether this employee is exempt from overtime pay.

    This classification is based on U.S. labor laws (FLSA).
    """

    pager: Optional[str] = None
    """The employee's pager number."""

    pager_pin: Optional[str] = FieldInfo(alias="pagerPin", default=None)
    """The employee's pager PIN."""

    phone: Optional[str] = None
    """The employee's primary telephone number."""

    print_as: Optional[str] = FieldInfo(alias="printAs", default=None)
    """The name to use when printing this employee from QuickBooks.

    By default, this is the same as the `name` field.
    """

    revision_number: str = FieldInfo(alias="revisionNumber")
    """
    The current QuickBooks-assigned revision number of this employee object, which
    changes each time the object is modified. When updating this object, you must
    provide the most recent `revisionNumber` to ensure you're working with the
    latest data; otherwise, the update will return an error.
    """

    salutation: Optional[str] = None
    """
    The employee's formal salutation title that precedes their name, such as "Mr.",
    "Ms.", or "Dr.".
    """

    ssn: Optional[str] = None
    """The employee's Social Security Number. The value can be with or without dashes.

    **NOTE**: This field cannot be changed after the employee is created.
    """

    supervisor: Optional[Supervisor] = None
    """The employee's supervisor.

    Found in the "employment job details" section of the employee's record in
    QuickBooks.
    """

    target_bonus: Optional[str] = FieldInfo(alias="targetBonus", default=None)
    """The target bonus for this employee, represented as a decimal string.

    Found in the "employment job details" section of the employee's record in
    QuickBooks.
    """

    termination_date: Optional[datetime.date] = FieldInfo(alias="terminationDate", default=None)
    """
    The date this employee's employment ended with the company, in ISO 8601 format
    (YYYY-MM-DD). This is also known as the released date or separation date.
    """

    updated_at: str = FieldInfo(alias="updatedAt")
    """
    The date and time when this employee was last updated, in ISO 8601 format
    (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time zone
    in QuickBooks.
    """

    us_citizenship_status: Optional[Literal["citizen", "non_citizen"]] = FieldInfo(
        alias="usCitizenshipStatus", default=None
    )
    """Indicates whether this employee is a U.S. citizen."""

    us_veteran_status: Optional[Literal["veteran", "non_veteran"]] = FieldInfo(alias="usVeteranStatus", default=None)
    """Indicates whether this employee is a U.S. veteran."""

    work_authorization_expiration_date: Optional[datetime.date] = FieldInfo(
        alias="workAuthorizationExpirationDate", default=None
    )
    """
    The date this employee's work authorization expires, in ISO 8601 format
    (YYYY-MM-DD).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "InventoryAdjustment",
    "Account",
    "Class",
    "Customer",
    "CustomField",
    "InventorySite",
    "Line",
    "LineInventorySiteLocation",
    "LineItem",
]


class Account(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class Class(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class Customer(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class CustomField(BaseModel):
    name: str
    """The name of the custom field, unique for the specified `ownerId`.

    For public custom fields, this name is visible as a label in the QuickBooks UI.
    """

    owner_id: str = FieldInfo(alias="ownerId")
    """
    The identifier of the owner of the custom field, which QuickBooks internally
    calls a "data extension". For public custom fields visible in the UI, such as
    those added by the QuickBooks user, this is always "0". For private custom
    fields that are only visible to the application that created them, this is a
    valid GUID identifying the owning application. Internally, Conductor always
    fetches all public custom fields (those with an `ownerId` of "0") for all
    objects.
    """

    type: Literal[
        "amount_type",
        "date_time_type",
        "integer_type",
        "percent_type",
        "price_type",
        "quantity_type",
        "string_1024_type",
        "string_255_type",
    ]
    """The data type of this custom field."""

    value: str
    """The value of this custom field.

    The maximum length depends on the field's data type.
    """


class InventorySite(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class LineInventorySiteLocation(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class LineItem(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class Line(BaseModel):
    id: str
    """The unique identifier assigned by QuickBooks to this inventory adjustment line.

    This ID is unique across all transaction line types.
    """

    expiration_date: Optional[date] = FieldInfo(alias="expirationDate", default=None)
    """
    The expiration date for the serial number or lot number of the item associated
    with this inventory adjustment line, in ISO 8601 format (YYYY-MM-DD). This is
    particularly relevant for perishable or time-sensitive inventory items. Note
    that this field is only supported on QuickBooks Desktop 2023 or later.
    """

    inventory_site_location: Optional[LineInventorySiteLocation] = FieldInfo(
        alias="inventorySiteLocation", default=None
    )
    """
    The specific location (e.g., bin or shelf) within the inventory site where the
    item associated with this inventory adjustment line is stored.
    """

    item: Optional[LineItem] = None
    """The inventory item associated with this inventory adjustment line."""

    lot_number: Optional[str] = FieldInfo(alias="lotNumber", default=None)
    """The lot number of the item associated with this inventory adjustment line.

    Used for tracking groups of inventory items that are purchased or manufactured
    together.
    """

    object_type: Literal["qbd_inventory_adjustment_line"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"qbd_inventory_adjustment_line"`."""

    quantity_difference: Optional[float] = FieldInfo(alias="quantityDifference", default=None)
    """
    Either a positive or negative number that shows the change in quantity for the
    inventory item associated with this inventory adjustment line. A positive number
    increases the quantity, while a negative number decreases it.
    """

    serial_number: Optional[str] = FieldInfo(alias="serialNumber", default=None)
    """The serial number of the item associated with this inventory adjustment line.

    This is used for tracking individual units of serialized inventory items.
    """

    serial_number_action: Optional[Literal["added", "removed"]] = FieldInfo(alias="serialNumberAction", default=None)
    """
    Indicates whether this inventory adjustment line's serial number was added or
    removed from the inventory.
    """

    value_difference: Optional[float] = FieldInfo(alias="valueDifference", default=None)
    """
    Either a positive or negative number that shows the change in the total value of
    the entire stock of the inventory item associated with this inventory adjustment
    line. A positive number increases the value, while a negative number decreases
    it.
    """


class InventoryAdjustment(BaseModel):
    id: str
    """The unique identifier assigned by QuickBooks to this inventory adjustment.

    This ID is unique across all transaction types.
    """

    account: Account
    """
    The account to which this inventory adjustment is posted for tracking inventory
    value changes.
    """

    class_: Optional[Class] = FieldInfo(alias="class", default=None)
    """The inventory adjustment's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default. A class defined here is automatically used in this inventory
    adjustment's line items unless overridden at the line item level.
    """

    created_at: str = FieldInfo(alias="createdAt")
    """
    The date and time when this inventory adjustment was created, in ISO 8601 format
    (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time zone
    in QuickBooks.
    """

    customer: Optional[Customer] = None
    """The customer or customer-job associated with this inventory adjustment."""

    custom_fields: List[CustomField] = FieldInfo(alias="customFields")
    """
    The custom fields for the inventory adjustment object, added as user-defined
    data extensions, not included in the standard QuickBooks object.
    """

    external_id: Optional[str] = FieldInfo(alias="externalId", default=None)
    """
    A globally unique identifier (GUID) you, the developer, can provide for tracking
    this object in your external system. This field is immutable and can only be set
    during object creation.
    """

    inventory_site: Optional[InventorySite] = FieldInfo(alias="inventorySite", default=None)
    """
    The site location where inventory for the item associated with this inventory
    adjustment is stored.
    """

    lines: List[Line]
    """
    The inventory adjustment's item lines, each representing the adjustment of an
    inventory item's quantity, value, serial number, or lot number.
    """

    memo: Optional[str] = None
    """A memo or note for this inventory adjustment."""

    object_type: Literal["qbd_inventory_adjustment"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"qbd_inventory_adjustment"`."""

    ref_number: Optional[str] = FieldInfo(alias="refNumber", default=None)
    """
    The case-sensitive user-defined reference number for this inventory adjustment,
    which can be used to identify the transaction in QuickBooks. This value is not
    required to be unique and can be arbitrarily changed by the QuickBooks user.
    """

    revision_number: str = FieldInfo(alias="revisionNumber")
    """
    The current QuickBooks-assigned revision number of this inventory adjustment
    object, which changes each time the object is modified. When updating this
    object, you must provide the most recent `revisionNumber` to ensure you're
    working with the latest data; otherwise, the update will return an error.
    """

    transaction_date: date = FieldInfo(alias="transactionDate")
    """The date of this inventory adjustment, in ISO 8601 format (YYYY-MM-DD)."""

    updated_at: str = FieldInfo(alias="updatedAt")
    """
    The date and time when this inventory adjustment was last updated, in ISO 8601
    format (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time
    zone in QuickBooks.
    """
